'use strict';

exports.__esModule = true;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

exports.parse = parse;
exports.stringify = stringify;

var _flkit = require('flkit');

var _stcHelper = require('stc-helper');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

//babylon can not use import
var babylon = null;
var babelGenerator = null;

/**
 * get attr value in attrs
 */
var getAttrValue = function getAttrValue(attrs, name) {
  var value = void 0;
  attrs.some(function (item) {
    if (item.nameLowerCase === name) {
      value = item.value;
      return true;
    }
  });
  return value;
};

/**
 * parse content
 */
var parseContent = function parseContent(parser, content, config) {
  var options = arguments.length <= 3 || arguments[3] === undefined ? {} : arguments[3];

  config = (0, _stcHelper.extend)({}, config);
  config.tpl = config.engine;
  var instance = new parser(content, config);
  if (config.adapter) {
    instance.registerTpl(config.engine, config.adapter);
  }
  for (var key in options) {
    instance[key] = options[key];
  }
  var tokens = instance.run();
  return tokens;
};

/**
 * parse html
 */
var parseHtml = function parseHtml(content, config, options) {
  var opts = (0, _stcHelper.extend)({}, config.tpl);
  opts.jsTplTypes = config.jsTpl && config.jsTpl.type;
  var tokens = parseContent(_flkit.HtmlTokenize, content, opts, options);
  tokens.forEach(function (token) {
    // tag start
    // parse style in tag
    // if(token.type === TokenType.HTML_TAG_START) {
    //   let attrs = token.ext.attrs;
    //   let styleValue = getAttrValue(attrs, 'style');
    //   if(styleValue){
    //     let tokens = parseCss(`*{${styleValue}}`, config, options);
    //     tokens = tokens.slice(2, tokens.length - 1);
    //     token.ext.styleTokens = tokens;
    //   }
    // }

    // style
    if (token.type === _flkit.TokenType.HTML_TAG_STYLE) {
      var contentToken = token.ext.content;
      var cssTokens = parseCss(contentToken.value, config.tpl, {
        line: contentToken.loc.start.line,
        col: contentToken.loc.start.col
      });
      contentToken.ext.tokens = cssTokens;
      return;
    }

    // js tpl
    if (token.type === _flkit.TokenType.HTML_TAG_SCRIPT) {
      var startToken = token.ext.start;
      if (startToken.ext.isTpl) {
        var _contentToken = token.ext.content;
        var htmlTokens = parseHtml(_contentToken.value, { tpl: config.jsTpl }, {
          line: _contentToken.loc.start.line,
          col: _contentToken.loc.start.col
        });
        _contentToken.ext.tokens = htmlTokens;
      }
    }
  });
  return tokens;
};

/**
 * parse css
 */
var parseCss = function parseCss(content, config, options) {
  config = (0, _extends3.default)({}, config);
  config.parseSelector = true;
  return parseContent(_flkit.CssTokenize, content, config, options);
};

/**
 * parse content to ast
 */
function parse(content, fileInstance, config) {
  var extname = fileInstance.extname.toLowerCase();
  switch (extname) {
    case 'js':
      if (!babylon) {
        babylon = require('babylon');
      }
      return babylon.parse(content).program;
    case 'css':
      return parseCss(content, config);
  }
  if (fileInstance.prop('tpl')) {
    return parseHtml(content, config);
  }
  throw new Error('file ' + fileInstance.path + ' can not get AST');
}

/**
 * stringify js
 */
var stringifyJS = function stringifyJS(ast, fileInstance) {
  if (!babelGenerator) {
    babelGenerator = require('babel-generator');
    if (babelGenerator.default) {
      babelGenerator = babelGenerator.default;
    }
  }
  var data = babelGenerator(ast, {
    comments: false,
    sourceMaps: false,
    filename: fileInstance && fileInstance.path || ''
  });
  return data.code;
};

/**
 * stringify css
 */
var stringifyCSS = function stringifyCSS(ast, fileInstance, delimiters) {
  return (0, _flkit.cssToken2Text)(ast, delimiters);
};

/**
 * convert ast to content
 */
function stringify(ast, fileInstance, config) {
  var extname = fileInstance.extname.toLowerCase();
  if (extname === 'js') {
    return stringifyJS(ast, fileInstance);
  }
  if (extname === 'css') {
    return stringifyCSS(ast, fileInstance);
  }
  if (fileInstance.prop('tpl')) {
    return (0, _flkit.htmlToken2Text)(ast, {
      js: stringifyJS,
      css: function css(ast) {
        var delimiters = [].concat(config.tpl.ld).concat(config.tpl.rd);
        return stringifyCSS(ast, fileInstance, delimiters);
      }
    });
  }
  throw new Error('can not convert file ' + fileInstance.path + ' AST to string');
}
//# sourceMappingURL=ast.js.map