'use strict';

exports.__esModule = true;
exports.worker = exports.master = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _stcPlugin = require('stc-plugin');

var _stcPlugin2 = _interopRequireDefault(_stcPlugin);

var _stcPluginInvoke = require('stc-plugin-invoke');

var _stcPluginInvoke2 = _interopRequireDefault(_stcPluginInvoke);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * special handle in cluster
 */
var master = exports.master = {
  /**
   * get file by path
   */
  // getFileByPath: (config, stc) => {
  //   let file = stc.resource.getFileByPath(config.file);
  //   return file.pathHistory;
  // },
  /**
   * get or set cache
   */
  cache: function cache(config, stc) {
    var key = config.key;
    var name = config.name;
    var value = config.value;

    if (!stc.cacheInstances[key]) {
      stc.cacheInstances[key] = new stc.cache({
        onlyMemory: true
      });
    }
    var instance = stc.cacheInstances[key];
    if (value === undefined) {
      return instance.get(name);
    }
    return instance.set(name, value);
  },
  /**
   * get file promise key & value
   */
  getFilePromise: function getFilePromise(config, stc) {
    var file = stc.resource.getFileByPath(config.file);
    var ret = file.promise(config.key);
    if (ret !== undefined) {
      return ret;
    }
    if (config.deferred) {
      file.promise(config.key, undefined, 'set');
    }
    return ret;
  },
  /**
   * resolve file deferred
   */
  resolveFilePromise: function resolveFilePromise(config, stc) {
    var file = stc.resource.getFileByPath(config.file);
    file.promise(config.key, config.value, 'update');
  },
  /**
   * add file
   */
  addFile: function addFile(config, stc) {
    stc.resource.addFile(config.file, config.content, config.virtual);
  },
  /**
   * return ast if have
   */
  getAstIfExist: function getAstIfExist(config, stc) {
    var file = stc.resource.getFileByPath(config.file);
    if (file.hasAst()) {
      return file.getAst();
    }
  },
  /**
   * update file ast
   */
  updateAst: function updateAst(config, stc) {
    var file = stc.resource.getFileByPath(config.file);
    file.setAst(config.ast);
  },
  /**
   * get file content
   */
  getContent: function () {
    var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(config, stc) {
      var file, content;
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              file = stc.resource.getFileByPath(config.file);
              _context.next = 3;
              return file.getContent(config.encoding);

            case 3:
              content = _context.sent;

              if (!Buffer.isBuffer(content)) {
                _context.next = 6;
                break;
              }

              return _context.abrupt('return', content.toString('base64'));

            case 6:
              return _context.abrupt('return', content);

            case 7:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, undefined);
    }));

    return function getContent(_x, _x2) {
      return _ref.apply(this, arguments);
    };
  }(),
  /**
   * exec plugin update method
   */
  update: function () {
    var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(config, stc) {
      var file, instance;
      return _regenerator2.default.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              file = stc.resource.getFileByPath(config.file);
              instance = new _stcPluginInvoke2.default(_stcPlugin2.default, file, {
                stc: stc,
                options: {}
              });

              instance.pluginInstance.prop('__isRun__', true);
              return _context2.abrupt('return', instance.pluginInstance.update(config.data));

            case 4:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, undefined);
    }));

    return function update(_x3, _x4) {
      return _ref2.apply(this, arguments);
    };
  }()
};
/**
 * special handle in worker
 */
var worker = exports.worker = {
  /**
   * get file ast
   */
  getAst: function getAst(config, stc) {
    var file = stc.resource.createFile(config.file, config.content);
    return file.getAst();
  }
};
//# sourceMappingURL=cluster_handle.js.map