'use strict';

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _stcHelper = require('stc-helper');

var _config = require('./config.js');

var _config2 = _interopRequireDefault(_config);

var _task = require('./task.js');

var _task2 = _interopRequireDefault(_task);

var _sourceMapSupport = require('source-map-support');

var _sourceMapSupport2 = _interopRequireDefault(_sourceMapSupport);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

require('babel-runtime/core-js/promise').default = require('bluebird');
global.Promise = require('bluebird');

_sourceMapSupport2.default.install({
  environment: 'node',
  emptyCacheBetweenOperations: true
});

/**
 * all configs
 */
var configs = {};
/**
 * Application class
 */
var Application = function () {
  /**
   * constructor
   */

  function Application() {
    (0, _classCallCheck3.default)(this, Application);

    this._env = '';
    this._config = null;

    this.env('default');
  }
  /**
   * get or set config
   */


  Application.prototype.config = function config(options) {
    if (!options) {
      return this._config.get();
    }
    this._config.set(options);
    return this;
  };
  /**
   * get or set env
   */


  Application.prototype.env = function env(_env) {
    if (_env) {
      if (!(_env in configs)) {
        configs[_env] = new _config2.default();
      }
      this._config = configs[_env];
      this._env = _env;
    } else {
      return this._env;
    }
    return this;
  };
  /**
   * translate config
   */


  Application.prototype.transConf = function transConf() {
    var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    if ((0, _stcHelper.isObject)(options)) {
      return (0, _keys2.default)(options).map(function (item) {
        var val = options[item];
        val.name = item;
        return val;
      });
    }
    return options;
  };
  /**
   * lint
   */


  Application.prototype.lint = function lint(options) {
    return this.config({ lint: this.transConf(options) });
  };
  /**
   * set transpile config
   */


  Application.prototype.transpile = function transpile(options) {
    return this.config({ transpile: this.transConf(options) });
  };
  /**
   * set dependence config
   */


  Application.prototype.dependence = function dependence() {
    var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    return this.config({ dependence: this.transConf(options) });
  };
  /**
   * set workflow config
   */


  Application.prototype.workflow = function workflow() {
    var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    return this.config({ workflow: this.transConf(options) });
  };
  /**
   * set preset config
   */


  Application.prototype.preset = function preset(_preset) {
    var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

    if (_preset.default && _preset.__esModule) {
      _preset = _preset.default;
    }
    options.transpile = this.transConf(options.transpile);
    options.dependence = this.transConf(options.dependence);
    options.workflow = this.transConf(options.workflow);

    _preset = (0, _stcHelper.extend)(_preset, options);
    return this.config(_preset);
  };
  /**
   * start workflow
   */


  Application.prototype.start = function start() {
    var config = this.config();
    if (!Array.isArray(config.tpl.ld)) {
      config.tpl.ld = [config.tpl.ld];
    }
    if (!Array.isArray(config.tpl.rd)) {
      config.tpl.rd = [config.tpl.rd];
    }

    var instance = new _task2.default(config);
    instance.run();
  };

  return Application;
}();

module.exports = new Application();
//# sourceMappingURL=index.js.map