'use strict';

exports.__esModule = true;

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _stcHelper = require('stc-helper');

var _stcFile = require('stc-file');

var _stcFile2 = _interopRequireDefault(_stcFile);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _cluster = require('cluster');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * look file cache
 */
var lookFileCache = {};

/**
 * get init files
 */
var getInitFiles = function getInitFiles(instance) {
  var files = [];
  var _instance$config = instance.config;
  var include = _instance$config.include;
  var exclude = _instance$config.exclude;
  var defaultExclude = _instance$config.defaultExclude;

  if (!(0, _stcHelper.isArray)(include)) {
    include = include ? [include] : [];
  }
  include.forEach(function (itemPath) {
    if (itemPath) {
      var tFiles = (0, _stcHelper.getFiles)(itemPath, itemPath);
      files = files.concat(tFiles);
    }
  });
  return files.filter(function (item) {
    return !instance.match(item, defaultExclude) && !instance.match(item, exclude);
  }).map(function (item) {
    var file = new _stcFile2.default({
      path: item,
      astHandle: instance.astHandle
    });
    if (instance.isTpl(item)) {
      file.prop('tpl', true);
    }
    return file;
  }).sort(function (a, b) {
    return a.path.split(_path2.default.sep).length > b.path.split(_path2.default.sep).length ? -1 : 1;
  });
};

/**
 * update file property
 */
var updateFile = function updateFile(file, content, virtual) {
  if (content === true) {
    virtual = true;
    content = undefined;
  }
  if (content !== undefined) {
    if ((0, _stcHelper.isString)(content) || (0, _stcHelper.isBuffer)(content)) {
      file.setContent(content);
    } else {
      file.setAst(content);
    }
  }
  if (virtual) {
    file.prop('virtual', true);
  }
  return file;
};

/**
 * get file by path
 */
var _getFileByPath = function _getFileByPath(instance, filepath) {
  var file = void 0;
  instance.files.some(function (item) {
    if (item.isPath(filepath)) {
      return file = item;
    }
  });
  if (file) {
    lookFileCache[filepath] = file;
    return file;
  }
};

/**
 * get file by path handle 
 */
var getFileByPathHandle = function getFileByPathHandle(instance, linkpath) {
  var filepath = instance.config.pathHandle(linkpath);
  if (filepath) {
    var file = void 0;
    instance.files.some(function (item) {
      if (item.isPath(filepath) > -1) {
        return file = item;
      }
    });
    if (file) {
      lookFileCache[linkpath] = file;
      return file;
    }
  }
};
/**
 * search File
 */
var searchFile = function searchFile(instance, linkpath) {
  // linkpath is longer than filepath
  var file = void 0;
  instance.files.some(function (item) {
    var pathHistory = item.pathHistory,
        length = pathHistory.length;
    for (var i = 0; i < length; i++) {
      var pos = pathHistory[i].indexOf(linkpath);
      if (pos > -1 && pathHistory[i].slice(pos) === linkpath) {
        file = item;
        return true;
      }
    }
  });
  if (file) {
    lookFileCache[linkpath] = file;
    return file;
  }
};

/**
 * look file
 */
var lookFile = function lookFile(instance, linkpath) {
  if (instance.config.pathHandle) {
    return getFileByPathHandle(instance, linkpath);
  }
  return searchFile(instance, linkpath);
};

/**
 * resource manage
 */

var Resource = function () {
  /**
   * constructor
   */

  function Resource() {
    var config = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
    var astHandle = arguments[1];
    (0, _classCallCheck3.default)(this, Resource);

    this.config = config;
    this.astHandle = astHandle;
    this.files = _cluster.isMaster ? getInitFiles(this) : [];
  }
  /**
   * test file match pattern
   */


  Resource.prototype.match = function match(file) {
    var _this = this;

    var pattern = arguments.length <= 1 || arguments[1] === undefined ? [] : arguments[1];

    if (!pattern) {
      return false;
    }
    if (!(0, _stcHelper.isArray)(pattern)) {
      pattern = [pattern];
    }
    var filepath = file.path || file;
    var matches = null;
    var flag = pattern.some(function (item) {
      // /\w/
      if ((0, _stcHelper.isRegExp)(item)) {
        matches = filepath.match(item);
        if (matches) {
          return true;
        }
      }
      if ((0, _stcHelper.isObject)(item)) {
        // {type: 'tpl'}
        if (item.type === 'tpl') {
          if (file.path) {
            return file.prop('tpl') === true;
          }
          return _this.isTpl(file);
        }
      }
      //@TODO support glob pattern?
      return item === filepath;
    });
    if (flag && !matches) {
      matches = [filepath];
    }
    return matches;
  };
  /**
   * get files
   */


  Resource.prototype.getFiles = function getFiles() {
    var _this2 = this;

    var include = arguments.length <= 0 || arguments[0] === undefined ? [] : arguments[0];
    var exclude = arguments.length <= 1 || arguments[1] === undefined ? [] : arguments[1];

    return this.files.filter(function (item) {
      if (item.prop('virtual')) {
        return false;
      }
      if (!!_this2.match(item, include) && !_this2.match(item, exclude)) {
        return true;
      }
    });
  };
  /**
   * check file is template file
   */


  Resource.prototype.isTpl = function isTpl(extname) {
    if (extname.indexOf('.') > -1) {
      extname = _path2.default.extname(extname).slice(1);
    }
    if (extname === 'html') {
      return true;
    }
    var exts = this.config.tpl.extname;
    if (!(0, _stcHelper.isArray)(exts)) {
      exts = [exts];
    }
    return exts.indexOf(extname.toLowerCase()) > -1;
  };
  /**
   * create file
   */


  Resource.prototype.createFile = function createFile(filepath, content, virtual) {
    if (filepath[0] === '/') {
      filepath = filepath.slice(1);
    }
    var instance = new _stcFile2.default({
      path: filepath,
      astHandle: this.astHandle
    });
    if (this.isTpl(filepath)) {
      instance.prop('tpl', true);
    }
    return updateFile(instance, content, virtual);
  };
  /**
   * add file
   */


  Resource.prototype.addFile = function addFile(filepath, content, virtual) {
    var file = void 0;
    this.files.some(function (item) {
      if (item.isPath(filepath)) {
        file = item;
        return true;
      }
    });
    if (file) {
      return updateFile(file, content, virtual);
    }
    var instance = this.createFile(filepath, content, virtual);
    this.files.push(instance);
    return instance;
  };
  /**
   * get file by path
   */


  Resource.prototype.getFileByPath = function getFileByPath(filepath, parentFile) {
    //is already stc-file instance
    if (!(0, _stcHelper.isString)(filepath) && filepath.path) {
      return filepath;
    }
    if (filepath[0] === '/') {
      filepath = filepath.slice(1);
    }
    if (lookFileCache[filepath]) {
      return lookFileCache[filepath];
    }
    var file = _getFileByPath(this, filepath);
    if (!file) {
      file = lookFile(this, filepath);
    }
    if (file) {
      return file;
    }
    var message = 'can not find file `' + filepath + '`';
    if (parentFile) {
      message += ' in `' + parentFile + '`';
    }
    throw new Error(message);
  };

  return Resource;
}();

exports.default = Resource;
//# sourceMappingURL=resource.js.map