'use strict';

exports.__esModule = true;

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _stcCluster = require('stc-cluster');

var _stcCluster2 = _interopRequireDefault(_stcCluster);

var _stcPlugin = require('stc-plugin');

var _stcPlugin2 = _interopRequireDefault(_stcPlugin);

var _stcPluginInvoke = require('stc-plugin-invoke');

var _stcPluginInvoke2 = _interopRequireDefault(_stcPluginInvoke);

var _stcCache = require('stc-cache');

var _stcCache2 = _interopRequireDefault(_stcCache);

var _stcLog = require('stc-log');

var _stcLog2 = _interopRequireDefault(_stcLog);

var _cluster = require('cluster');

var _stcHelper = require('stc-helper');

var _ast = require('./ast.js');

var _resource = require('./resource.js');

var _resource2 = _interopRequireDefault(_resource);

var _flkit = require('flkit');

var flkit = _interopRequireWildcard(_flkit);

var _cluster_handle = require('./cluster_handle.js');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var clusterLog = (0, _debug2.default)('cluster');
var pluginFileTime = (0, _debug2.default)('pluginFileTime');

/**
 * STC class
 */

var STC = function () {
  /**
   * constructor
   */

  function STC(config) {
    (0, _classCallCheck3.default)(this, STC);

    this.config = config;
    this.resource = this.getResourceInstance();
    this.cluster = this.getClusterInstance();
    this.cache = _stcCache2.default; // cache class
    // store all cache instances
    this.cacheInstances = {};
    // flkit
    this.flkit = flkit;
    this.flkit.getFlkitInstance = this.getFlkitInstance.bind(this);
    this.log = new _stcLog2.default();
    this.debug = _debug2.default;
  }
  /**
   * get flkit instance
   */


  STC.prototype.getFlkitInstance = function getFlkitInstance(type, text) {
    var options = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

    if (options.engine && !options.tpl) {
      options.tpl = options.engine;
    }
    var cls = this.flkit[type];
    options = (0, _stcHelper.extend)({}, this.config.tpl, options);
    var instance = new cls(text, options);
    if (options.adapter) {
      instance.registerTpl(options.adapter);
    }
    return instance;
  };
  /**
   * get cluster instance
   */


  STC.prototype.getClusterInstance = function getClusterInstance() {
    var instance = new _stcCluster2.default({
      workers: this.config.workers,
      workerHandle: this.workerHandle.bind(this),
      masterHandle: this.masterHandle.bind(this),
      logger: clusterLog
    });
    if (this.config.cluster !== false) {
      instance.start();
    }
    return instance;
  };
  /**
   * get resource instance
   */


  STC.prototype.getResourceInstance = function getResourceInstance() {
    var _this = this;

    var instance = new _resource2.default(this.config, {
      parse: function parse() {
        for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        return _ast.parse.apply(undefined, args.concat([_this.config]));
      },
      stringify: function stringify() {
        for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
          args[_key2] = arguments[_key2];
        }

        return _ast.stringify.apply(undefined, args.concat([_this.config]));
      }
    });
    return instance;
  };
  /**
   * invoked in master
   */


  STC.prototype.masterHandle = function masterHandle(config) {
    var method = config.method;
    var args = config.args;
    var options = config.options;
    var file = config.file;


    if (_cluster_handle.master[method]) {
      return _cluster_handle.master[method](config, this);
    }

    file = this.resource.getFileByPath(file);
    var instance = new _stcPluginInvoke2.default(_stcPlugin2.default, file, {
      stc: this,
      options: options
    });
    return instance.invokePluginMethod(method, args);
  };
  /**
   * invoked in worker
   */


  STC.prototype.workerHandle = function workerHandle(config) {
    var type = config.type;
    var pluginIndex = config.pluginIndex;
    var file = config.file;


    if (_cluster_handle.worker[type]) {
      return _cluster_handle.worker[type](config, this);
    }

    //invoke plugin
    var opts = this.config[type][pluginIndex];
    if (!opts) {
      throw new Error('plugin not found, type: ' + type + ', pluginIndex: ' + pluginIndex);
    }

    var ie = this.getIncludeAndExclude(opts);
    file = this.resource.createFile(file);
    var options = opts.options;
    var cluster = opts.cluster;
    var cache = opts.cache;

    var instance = new _stcPluginInvoke2.default(opts.plugin, file, {
      stc: this,
      options: options,
      include: ie.include,
      cluster: cluster,
      cache: cache,
      logger: pluginFileTime,
      ext: {
        type: type,
        pluginIndex: pluginIndex
      }
    });
    return instance.run();
  };
  /**
   * get include and exclude
   */


  STC.prototype.getIncludeAndExclude = function getIncludeAndExclude(pluginOptions) {
    var include = pluginOptions.include;
    var exclude = pluginOptions.exclude;

    var pluginClass = _stcPluginInvoke2.default.getPluginClass(pluginOptions.plugin);
    if (!include) {
      include = pluginClass.include;
      if (typeof include === 'function') {
        include = include();
      }
    }
    if (!exclude) {
      exclude = pluginClass.exclude;
      if (typeof exclude === 'function') {
        exclude = exclude();
      }
    }
    return { include: include, exclude: exclude };
  };

  return STC;
}();

exports.default = STC;
//# sourceMappingURL=stc.js.map