'use strict';

exports.__esModule = true;

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _cluster = require('cluster');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _stcPluginInvoke = require('stc-plugin-invoke');

var _stcPluginInvoke2 = _interopRequireDefault(_stcPluginInvoke);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _stcHelper = require('stc-helper');

var _stc = require('./stc.js');

var _stc2 = _interopRequireDefault(_stc);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var pluginFilesLog = (0, _debug2.default)('pluginMatchedFiles');
var pluginFileTime = (0, _debug2.default)('pluginFileTime');
var pluginTime = (0, _debug2.default)('pluginTotalTime');

/**
 * Task class
 */

var Task = function () {
  /**
   * constructor
   */

  function Task(config) {
    (0, _classCallCheck3.default)(this, Task);

    this.config = config;
    this.stc = new _stc2.default(config);
  }
  /**
   * run plugin task
   */


  Task.prototype.runPlugin = function () {
    var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(pluginOptions, ext) {
      var pluginClass, ie, files, pluginName, consoleFiles, startTime, options, cluster, cache, ret, endTime;
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              if (!(pluginOptions.on === false)) {
                _context.next = 2;
                break;
              }

              return _context.abrupt('return');

            case 2:
              pluginClass = _stcPluginInvoke2.default.getPluginClass(pluginOptions.plugin);
              ie = this.stc.getIncludeAndExclude(pluginOptions);
              //no files matched

              files = this.stc.resource.getFiles(ie.include, ie.exclude);

              if (files.length) {
                _context.next = 7;
                break;
              }

              return _context.abrupt('return');

            case 7:
              pluginName = pluginClass.name;
              consoleFiles = (0, _stringify2.default)(files.map(function (file) {
                return file.path;
              }));

              pluginFilesLog(pluginName + ': length=' + files.length + ', files=' + consoleFiles);

              startTime = Date.now();
              options = pluginOptions.options;
              cluster = pluginOptions.cluster;
              cache = pluginOptions.cache;
              _context.next = 16;
              return _stcPluginInvoke2.default.run(pluginClass, files, {
                stc: this.stc,
                options: options,
                include: ie.include,
                cluster: cluster,
                cache: cache,
                logger: pluginFileTime,
                ext: ext
              });

            case 16:
              ret = _context.sent;
              endTime = Date.now();

              pluginTime(pluginName + ': files=' + files.length + ', time=' + (endTime - startTime) + 'ms');

              return _context.abrupt('return', ret);

            case 20:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    function runPlugin(_x, _x2) {
      return _ref.apply(this, arguments);
    }

    return runPlugin;
  }();
  /**
   * parallel to execute
   */


  Task.prototype.parallel = function parallel(type) {
    var _this = this;

    var config = this.config[type];
    if (!config) {
      return;
    }
    var promises = config.map(function (item, pluginIndex) {
      return _this.runPlugin(item, {
        type: type,
        pluginIndex: pluginIndex
      });
    });
    return _promise2.default.all(promises);
  };
  /**
   * serial to execute
   */


  Task.prototype.serial = function () {
    var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(type) {
      var config, length, i;
      return _regenerator2.default.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              config = this.config[type];

              if (config) {
                _context2.next = 3;
                break;
              }

              return _context2.abrupt('return');

            case 3:
              length = config.length;
              i = 0;

            case 5:
              if (!(i < length)) {
                _context2.next = 11;
                break;
              }

              _context2.next = 8;
              return this.runPlugin(config[i], {
                type: type,
                pluginIndex: i
              });

            case 8:
              i++;
              _context2.next = 5;
              break;

            case 11:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, this);
    }));

    function serial(_x3) {
      return _ref2.apply(this, arguments);
    }

    return serial;
  }();
  /**
   * output files
   */


  Task.prototype.output = function output() {
    var _this2 = this;

    var outputPath = this.config.outputPath;
    var files = this.stc.resource.files;
    var promises = files.map(function () {
      var _ref3 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3(file) {
        var savePath, content, writeFile;
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                if (!file.prop('virtual')) {
                  _context3.next = 2;
                  break;
                }

                return _context3.abrupt('return');

              case 2:
                savePath = _path2.default.join(outputPath, file.path);

                (0, _stcHelper.mkdir)(_path2.default.dirname(savePath));
                _context3.next = 6;
                return file.getContent();

              case 6:
                content = _context3.sent;
                writeFile = (0, _stcHelper.promisify)(_fs2.default.writeFile, _fs2.default);
                return _context3.abrupt('return', writeFile(savePath, content));

              case 9:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, _this2);
      }));

      return function (_x4) {
        return _ref3.apply(this, arguments);
      };
    }());
    return _promise2.default.all(promises);
  };
  /**
   * run
   */


  Task.prototype.run = function () {
    var _ref4 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee4() {
      var startTime, endTime;
      return _regenerator2.default.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              if (!_cluster.isMaster) {
                _context4.next = 25;
                break;
              }

              startTime = Date.now();
              _context4.prev = 2;
              _context4.next = 5;
              return this.parallel('lint');

            case 5:
              _context4.next = 7;
              return this.parallel('transpile');

            case 7:
              _context4.next = 9;
              return this.parallel('dependence');

            case 9:
              _context4.next = 11;
              return this.serial('workflow');

            case 11:
              _context4.next = 13;
              return this.output();

            case 13:
              _context4.next = 21;
              break;

            case 15:
              _context4.prev = 15;
              _context4.t0 = _context4['catch'](2);

              // error message is json data
              if (_context4.t0.message[_context4.t0.message.length - 1] === '}') {
                try {
                  _context4.t0 = JSON.parse(_context4.t0.message.slice(_context4.t0.message.indexOf('{')));
                } catch (e) {}
              }

              if (_context4.t0.className) {
                this.stc.log.display(_context4.t0, 'error');
              } else {
                console.error(_context4.t0 && _context4.t0.stack || _context4.t0);
              }
              process.exit(100);
              return _context4.abrupt('return');

            case 21:
              this.stc.cluster.stop();
              endTime = Date.now();

              console.log('Build finish, Total time: ' + (endTime - startTime) + 'ms');
              process.exit(0);

            case 25:
            case 'end':
              return _context4.stop();
          }
        }
      }, _callee4, this, [[2, 15]]);
    }));

    function run() {
      return _ref4.apply(this, arguments);
    }

    return run;
  }();

  return Task;
}();

exports.default = Task;
//# sourceMappingURL=task.js.map